#' Compute forecast
#'
#' Predict time-series curves for the selected days indices (lines in data).
#'
#' @param data Dataset, object of type \code{Data} output of \code{getData}
#' @param indices Days indices where to forecast (the day after)
#' @param forecaster Name of the main forcaster
#' \itemize{
#'   \item Persistence : use values of last (similar, next) day
#'   \item Neighbors : use values from the k closest neighbors' tomorrows
#'   \item Average : global average of all the (similar) "tomorrow of past"
#'   \item Zero : just output 0 (benchmarking purpose)
#' }
#' @param pjump How to predict the jump at the interface between two days ?
#' \itemize{
#'   \item Persistence : use last (similar) day values
#'   \item Neighbors: re-use the weights optimized in corresponding forecaster
#'   \item Zero: just output 0 (no adjustment)
#' }
#' @param memory Data depth (in days) to be used for prediction
#' @param horizon Number of time steps to predict
#' @param ... Additional parameters for the forecasting models
#'
#' @return An object of class Forecast
#'
#' @examples
#' ts_data = system.file("extdata","pm10_mesures_H_loc.csv",package="talweg")
#' exo_data = system.file("extdata","meteo_extra_noNAs.csv",package="talweg")
#' data = getData(ts_data, exo_data, input_tz = "Europe/Paris",
#'   working_tz="Europe/Paris", predict_at=7)
#' pred = computeForecast(data, 2200:2230, "Persistence", "Persistence", 500, 12)
#' \dontrun{#Sketch for real-time mode:
#' data = new("Data", ...)
#' forecaster = new(..., data=data)
#' repeat {
#'   data$append(some_new_data)
#'   pred = forecaster$predict(data$getSize(), ...)
#'   #do_something_with_pred
#' }}
#' @export
computeForecast = function(data, indices, forecaster, pjump,
	memory=Inf, horizon=data$getStdHorizon(), ...)
{
	# (basic) Arguments sanity checks
	horizon = as.integer(horizon)[1]
	if (horizon<=0 || horizon>length(data$getCenteredSerie(2)))
		stop("Horizon too short or too long")
	integer_indices = sapply(indices, function(i) dateIndexToInteger(i,data))
	if (any(integer_indices<=0 | integer_indices>data$getSize()))
		stop("Indices out of range")
	if (!is.character(forecaster) || !is.character(pjump))
		stop("forecaster (name) and pjump (function) should be of class character")

	pred = Forecast$new( sapply(indices, function(i) integerIndexToDate(i,data)) )
	forecaster_class_name = getFromNamespace(paste(forecaster,"Forecaster",sep=""), "talweg")
	forecaster = forecaster_class_name$new( #.pjump =
		getFromNamespace(paste("get",pjump,"JumpPredict",sep=""), "talweg"))
	for (today in integer_indices)
	{
		pred$append(
			new_serie = forecaster$predictSerie(data, today, memory, horizon, ...),
			new_params = forecaster$getParameters(),
			new_index_in_data = today
		)
	}
	pred
}
